increasing.pop.size.ggplot <-
  function(initial.population.a=1, initial.population.b=100, initial.population.c=100000, maxU=20, aspect=2){
    require(ggplot2)
    path <- file.path("..","data","extn_surface-long")
    analytic.extn.surf <- as.matrix(read.csv(path, header=FALSE))
    x.indices.to.exclude <- 75:100
    x.an <- maxU * seq(0.01, 1, 0.01)[-x.indices.to.exclude]
    y <- seq(0.01, 1, 0.01)
    z.an <- analytic.extn.surf[seq(100, 1, -1)[-x.indices.to.exclude],]
    postscript(file="increasing-pop-size-ggplot.eps", width=6.83, height=3.27, horizontal=FALSE, onefile=FALSE, paper="special")

    M <- melt(z.an^initial.population.a)
    D <- cbind(expand.grid(x.an,y),M)
    colnames(D) <- c("U", "s", "Uf", "sl", "p")
    D$comp.s <- 1 - D$s
    p <- ggplot(D, aes(x=U, y=comp.s, fill=p))
    pg <- p+geom_tile()+ labs(x="Mutation rate U", y="Initial robustness  1-p")
    pg <- pg + theme_bw() + opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(), panel.background=theme_blank(), legend.position="none", aspect.ratio=aspect)
    panel.A <- pg + scale_fill_gradient2()+ scale_x_continuous(breaks=seq(2,14,4))


    M <- melt(z.an^initial.population.b)
    D <- cbind(expand.grid(x.an,y),M)
    colnames(D) <- c("U", "s", "Uf", "sl", "p")
    D$comp.s <- 1 - D$s
    p <- ggplot(D, aes(x=U, y=comp.s, fill=p))
    pg <- p+geom_tile()+ labs(x="Mutation rate U", y="Initial robustness  1-p")
    pg <- pg + theme_bw() + opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(), panel.background=theme_blank(), legend.position="none", aspect.ratio=aspect)
    panel.B <- pg + scale_fill_gradient2() + scale_x_continuous(breaks=seq(2,14,4))


    
    M <- melt(z.an^initial.population.c)
    D <- cbind(expand.grid(x.an,y),M)
    colnames(D) <- c("U", "s", "Uf", "sl", "p")
    D$comp.s <- 1 - D$s
    p <- ggplot(D, aes(x=U, y=comp.s, fill=p))
    pg <- p+geom_tile()+ labs(x="Mutation rate U", y="Initial robustness  1-p", fill="Pr(extinction)")
    pg <- pg + theme_bw() + opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(), panel.background=theme_blank(), legend.key=theme_blank(), aspect.ratio=aspect)
    panel.C <- pg + scale_fill_gradient2() + scale_x_continuous(breaks=seq(2,14,4))

    grid.newpage()
    pushViewport(viewport(layout=grid.layout(1,3, widths=c(3,3,4.5))))

    vplayout <- function(x, y){
      viewport(layout.pos.row = x, layout.pos.col = y)
    }
    print(panel.A, vp = vplayout(1,1))
    print(panel.B, vp = vplayout(1,2))
    print(panel.C, vp = vplayout(1,3))

    labheight <- .95
    grid.text("A", .03, labheight, gp=gpar(fontsize=12, font=2))
    grid.text("B", .31, labheight, gp=gpar(fontsize=12, font=2))
    grid.text("C", .6, labheight, gp=gpar(fontsize=12, font=2))
    dev.off()
  }

