idef = 1 
n = 20 
bdef = 20
sigma = 100
kdef = 8
A = 4
ndef = 10000
prec = 512 

def mu_tilde(q):
    return (1-q)/((A-1)*q)

def fit_eq(q, i=idef,b=bdef, k=kdef):
    l = 2*b*n + k*(b-1)
    return sigma*q^(l)*(1+((mu_tilde(q))^2)/2 + mu_tilde(q)/2*sqrt(8+(mu_tilde(q))^2))^(n*i)

def min_f(q, i=idef, b=bdef, k=kdef):
    return abs(1 - fit_eq(q, i, b, k))

def fix_prob(q, i=idef, b=bdef, k=kdef):
    return 2*(fit_eq(q, i+1,b, k)/fit_eq(q, i,b, k) - 1)

def prob_cre(q, i=idef, b=bdef, k=kdef, N=ndef):
    return  1 - (1 - fit_eq(q,i,b, k)/sigma * (mu_tilde(q))^k * q^(-2 * n) * (1 - (q^2) * (mu_tilde(q))^2 )^n )^(N*fit_eq(q,i,b, k))

def tau(q, i=idef, b=bdef, k=kdef, N=ndef):
#    print fix_prob(q, i, b, k), prob_cre(q, i, b, k, N)
    return 1/(fix_prob(q, i, b, k) * prob_cre(q, i, b, k, N))

def get_con_time(U, b=bdef, k=kdef, N=ndef):
    l = 2*b*n + k*(b-1)
    q = 1 - U/l
    t = 0
    for j in range(1, b):
        t = t + tau(q, j, b, k, N)
    return t

def get_con_series(b=bdef, k=kdef, numpoints=100, upper=6, N=ndef):
    muts = []
    times = []
    for p in range(1, numpoints + 1):
        U = (1e-5 + p * (upper - 1e-5)/numpoints).n(prec)
        muts.append(U)
        times.append(get_con_time(U, b, k, N))
    return muts, times

def get_ucrit_series(i = idef, bmax=50, k=kdef):
    ucrit = []
    bser = []
    for b in range(i, bmax + 1):
        l = 2*b*n + k*(b-1)
        bser.append(b)
        def f(q):
            return abs(1 - fit_eq(q, i=i, b=b, k=k))
        u = l * (1-find_minimum_on_interval(f, 0, 1)[1])
        ucrit.append(u)
    return bser, ucrit

def get_i_series(i = (1, 10, 20, 30, 40, 50), k=kdef):
    vec = []
    for j in range(len(i)):
        vec.append(get_ucrit_series(i=i[j], k=k))
    return vec

def make_data_file(ans, filename='time_U.dat', header=1):
    f = open (filename, 'w')
    if header == 1:
        f.write('U contime\n')
    elif header == 2:
        f.write('b ucrit\n')
    for u,t in zip(ans[0], ans[1]):
        f.write('%.3f %.3f\n' % (u, t))
    f.close()

def make_db_data_file(ans, filename='time_U.dat', b=bdef, k=kdef, N=ndef):
    f = open (filename, 'w')
    f.write('U contime b k N\n')
    for u,t in zip(ans[0], ans[1]):
        f.write('%.3f %.3f %.3f %.3f %d\n' % (u, t, b, k, N))
    f.close()

out = get_i_series()
make_data_file(filename='ucrit_v_b10.dat', header=2, ans=out[1])
make_data_file(filename='ucrit_v_b1.dat', header=2, ans=out[0])
make_data_file(filename='ucrit_v_b20.dat', header=2, ans=out[2])
make_data_file(filename='ucrit_v_b30.dat', header=2, ans=out[3])
make_data_file(filename='ucrit_v_b40.dat', header=2, ans=out[4])
make_data_file(filename='ucrit_v_b50.dat', header=2, ans=out[5])

ans=get_con_series(k=2,upper=10)
make_db_data_file(filename='time_U_k2_Ne4.dat', ans=ans, k=2)
ans=get_con_series(k=4,upper=10)
make_db_data_file(filename='time_U_k4_Ne4.dat', ans=ans, k=4)
ans=get_con_series(k=6, upper=10)
make_db_data_file(filename='time_U_k6_Ne4.dat', ans=ans, k=6)
ans=get_con_series(k=8, upper=10)
make_db_data_file(filename='time_U_k8_Ne4.dat', ans=ans, k=8)
"""
ans=get_con_series(k=2,upper=10, N=100000)
make_db_data_file(filename='time_U_k2_Ne5.dat', ans=ans, k=2, N=100000)
ans=get_con_series(k=4,upper=10, N=100000)
make_db_data_file(filename='time_U_k4_Ne5.dat', ans=ans, k=4, N=100000)
ans=get_con_series(k=6, upper=10, N=100000)
make_db_data_file(filename='time_U_k6_Ne5.dat', ans=ans, k=6, N=100000)
ans=get_con_series(k=8, upper=10, N=100000)
make_db_data_file(filename='time_U_k8_Ne5.dat', ans=ans, k=8, N=100000)

ans=get_con_series(k=2,upper=10, N=1000000)
make_db_data_file(filename='time_U_k2_Ne6.dat', ans=ans, k=2, N=1000000)
ans=get_con_series(k=4,upper=10, N=1000000)
make_db_data_file(filename='time_U_k4_Ne6.dat', ans=ans, k=4, N=1000000)
ans=get_con_series(k=6, upper=10, N=1000000)
make_db_data_file(filename='time_U_k6_Ne6.dat', ans=ans, k=6, N=1000000)
ans=get_con_series(k=8, upper=10, N=1000000)
make_db_data_file(filename='time_U_k8_Ne6.dat', ans=ans, k=8, N=1000000)

"""
ans=get_con_series(k=2,upper=10, N=100000000)
make_db_data_file(filename='time_U_k2_Ne8.dat', ans=ans, k=2, N=100000000)
ans=get_con_series(k=4,upper=10, N=100000000)
make_db_data_file(filename='time_U_k4_Ne8.dat', ans=ans, k=4, N=100000000)
ans=get_con_series(k=6, upper=10, N=100000000)
make_db_data_file(filename='time_U_k6_Ne8.dat', ans=ans, k=6, N=100000000)
ans=get_con_series(k=8, upper=10, N=100000000)
make_db_data_file(filename='time_U_k8_Ne8.dat', ans=ans, k=8, N=100000000)
"""
ans=get_con_series(k=2,upper=10, N=100000000000)
make_db_data_file(filename='time_U_k2_Ne11.dat', ans=ans, k=2, N=100000000000)
ans=get_con_series(k=4,upper=10, N=100000000000)
make_db_data_file(filename='time_U_k4_Ne11.dat', ans=ans, k=4, N=100000000000)
ans=get_con_series(k=6, upper=10, N=100000000000)
make_db_data_file(filename='time_U_k6_Ne11.dat', ans=ans, k=6, N=100000000000)
ans=get_con_series(k=8, upper=10, N=100000000000)
make_db_data_file(filename='time_U_k8_Ne11.dat', ans=ans, k=8, N=100000000000)
"""
ans=get_con_series(k=2,upper=10, N=1000000000000)
make_db_data_file(filename='time_U_k2_Ne12.dat', ans=ans, k=2, N=1000000000000)
ans=get_con_series(k=4,upper=10, N=1000000000000)
make_db_data_file(filename='time_U_k4_Ne12.dat', ans=ans, k=4, N=1000000000000)
ans=get_con_series(k=6, upper=10, N=1000000000000)
make_db_data_file(filename='time_U_k6_Ne12.dat', ans=ans, k=6, N=1000000000000)
ans=get_con_series(k=8, upper=10, N=1000000000000)
make_db_data_file(filename='time_U_k8_Ne12.dat', ans=ans, k=8, N=1000000000000)
