rna.and.analytic.ggplot <-
  function(maxU=20){
    require(ggplot2) 
    path <- file.path("..","data","extn_surf_rna-and-analytic.csv")
    path2 <- file.path("..","data","ext-logCalN34.csv")
    analytic.extn.surf <- as.matrix(read.csv(path, header=FALSE))
    source("smoothing-functions.R")
    rna.extn.surf <- as.matrix(smoother(path2,3,2))
    x.rna <- seq(0.1, maxU, 0.1)
    rna.extn.surf <- cbind(rna.extn.surf, matrix(1, nrow=dim(rna.extn.surf)[1], ncol=length(x.rna)-dim(rna.extn.surf)[2]))
    x.an <- maxU * seq(0.01, 1, 0.01)
    y.indices.to.exclude <- c(1:19, 81:100)
    y <- seq(0.01, 1, 0.01)[-y.indices.to.exclude]
    z.an <- analytic.extn.surf[seq(100,1,-1), -y.indices.to.exclude]
    z.rna <- t(rna.extn.surf)
    postscript(file="rna-and-analytic-ggplot.eps", width=6.83, height=3.27, horizontal=FALSE, onefile=FALSE, paper="special")
    
    M <- melt(z.rna)
    D <- cbind(expand.grid(x.rna,y),M)
    colnames(D) <- c("U", "s", "Uf", "sl", "p")
    D$comp.s <- 1 - D$s

    p <- ggplot(D, aes(x=U, y=comp.s, fill=p))
    pg <- p+geom_tile()+ labs(x="Mutation rate U", y="Initial robustness  1-p")
    pg <- pg + theme_bw() + opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(), panel.background=theme_blank(), legend.position="none", aspect.ratio=1)
    panel.A <- pg + scale_fill_gradient2()

    M <- melt(z.an)
    D <- cbind(expand.grid(x.an, y), M)
    colnames(D) <- c("U", "s", "Uf", "sl", "p")
    D$comp.s <- 1 - D$s
    p <- ggplot(D, aes(x=U, y=comp.s, fill=p))
    pg <- p+geom_tile()+ labs(x="Mutation rate U", y="Initial robustness  1-p", fill="Pr(extinction)")
    pg <- pg + scale_fill_gradient2()
    pg <- pg + theme_bw() + opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(), panel.background=theme_blank(), legend.key=theme_blank(), aspect.ratio=1)
    panel.B <- pg

    grid.newpage()
    pushViewport(viewport(layout=grid.layout(1,2, widths=c(3,4))))

    vplayout <- function(x, y){
      viewport(layout.pos.row = x, layout.pos.col = y)
    }
    print(panel.A, vp = vplayout(1,1))
    print(panel.B, vp = vplayout(1,2))
    labheight <- .9
    grid.text("A", .03, labheight, gp=gpar(fontsize=12, font=2))
    grid.text("B", .455, labheight, gp=gpar(fontsize=12, font=2))
    dev.off()

  }

