"""
Copyright (c) 2009, Thomas Keller <thomas.e.keller [at] gmail.com>

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
# -*- coding: utf-8 -*-
import numpy as np
import sys
from pyrna_rob_final import *
import globs_rob_final as globs

def getfit(shape):
    if globs.fitfun is 'h':
        fitness=1.0/(globs.alpha+(float(hamming(shape,globs.target))/globs.glength)**globs.beta)
    elif globs.fitfun is 'l':
        fitness=globs.lin_scale-globs.lin_scale*hamming(shape,globs.target)*globs.lin_coeff/globs.glength
        if fitness<0.0:
            fitness=0.0
    return fitness
    
class genotype(object):
    def __init__(self,gtype,id,count,par_id,par_fit,shp=None,fit=None):
        self.p_id=par_id
        self.id=id
        if shp:
            self.shape=shp
        else:
            self.shape=pyfold(gtype)
        self.prev_copies=0
        self.cur_copies=count
        if fit:
            self.fitness=fit
        else:
            self.fitness=getfit(self.shape)
        self.sel_coeff=(self.fitness-par_fit)/par_fit if par_fit else 0


class population(object):
    def __init__(self,init_gtypes=None):
        self.num_new_seqs=1
        self.pop=dict()
        self.mean_fit=0.0
        self.gtype_in_pop=0
        self.gtypes=[]
        self.offspringvect=[]
        self.totoffspring=0
        if init_gtypes:
            for gtype,num_copies in init_gtypes:
                self.pop[gtype]=genotype(gtype,self.num_new_seqs,int(num_copies),0,0)
                self.num_new_seqs+=1
        else:
            for _ in range(popsize):
                gtype=[globs.bases[np.random.randint(0,4)] for _ in range(globs.glength)]
                gtype=''.join(gtype)
                while gtype in self.pop:
                    gtype=[globs.bases[np.random.randint(0,4)] for _ in range(globs.glength)]
                    gtype=''.join(gtype)
                self.pop[gtype]=genotype(gtype,self.num_new_seqs,1,0,0)
                self.num_new_seqs+=1
    
        
        
    def add_to_next_gen(self,indiv,recparent=None):
        """
        ultimate worker function, goes through and mutates given individual.
        """
        mutseq=mutc(indiv)
        if mutseq:
            if mutseq in self.pop:
                self.pop[mutseq].cur_copies+=1
            else:
                self.pop[mutseq]=genotype(mutseq,self.num_new_seqs,1,
                                            self.pop[indiv].id,self.pop[indiv].fitness)
                self.num_new_seqs+=1
            self.mean_fit+=self.pop[mutseq].fitness
        else:
            
            self.pop[indiv].cur_copies+=1
            self.mean_fit+=self.pop[indiv].fitness


    def nextgen(self):
        """
        basic function that does selection and replication over one generation. First number of offspring
        each parent genotype produces is calculated (for loop with np.random.poisson). If this number exceeds popcap, randomly choose offspring to represent next generation without replacement. Picked offspring mutate with per-site rate 1-fidelity. Otherwise, all offspring go into next generation. Regardless, various population statistics calculated every gen.
        """
        self.gtypes=self.pop.keys()
        self.totoffspring=0
        self.mean_fit=0.0
        self.rec_chamber=False
        for gtype in self.gtypes:
            self.pop[gtype].prev_copies=np.random.poisson(self.pop[gtype].cur_copies*globs.rep_val* \
                                                         self.pop[gtype].fitness/100.) if self.pop[gtype].fitness else 0
            self.pop[gtype].cur_copies=0
            self.totoffspring+=self.pop[gtype].prev_copies
        if not self.totoffspring:
            self.mean_fit=None
            return
        elif self.totoffspring>globs.popsize:
            return
        self.offspringvec=np.array([self.pop[gtype].prev_copies for gtype in self.gtypes],dtype=np.int32)
        osveclen=len(self.offspringvec)
        maxsize=self.totoffspring
        for rep_event in range(min(globs.popsize,self.totoffspring)):
            i=choose_finite(self.offspringvec,maxsize,osveclen)
            indiv=self.gtypes[i]
            self.offspringvec[i]-=1
            maxsize-=1
            self.add_to_next_gen(indiv)
        self.mean_fit /=min(globs.popsize,self.totoffspring)
        for gtype in self.gtypes:
            if not self.pop[gtype].cur_copies:
                self.pop.pop(gtype)

        self.gtype_in_pop=len(self.pop)
